# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing
from .customer_specific_reranker import CustomerSpecificReranker
from .user_function_reranker import UserFunctionReranker
from .mmr_reranker import MmrReranker
from .none_reranker import NoneReranker
import typing

if typing.TYPE_CHECKING:
    from .chain_reranker import ChainReranker
SearchReranker = typing.Union[
    CustomerSpecificReranker, UserFunctionReranker, MmrReranker, "ChainReranker", NoneReranker
]
