# This file was auto-generated by Fern from our API Definition.

import typing

Language = typing.Union[
    typing.Literal[
        "auto",
        "eng",
        "deu",
        "fra",
        "zho",
        "kor",
        "ara",
        "rus",
        "tha",
        "nld",
        "ita",
        "por",
        "spa",
        "jpn",
        "pol",
        "tur",
        "vie",
        "ind",
        "ces",
        "ukr",
        "ell",
        "heb",
        "fas",
        "hin",
        "urd",
        "swe",
        "ben",
        "msa",
        "ron",
    ],
    typing.Any,
]
