# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .query_history_summary import QueryHistorySummary
from .list_query_histories_response_metadata import ListQueryHistoriesResponseMetadata
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListQueryHistoriesResponse(UniversalBaseModel):
    """
    An array of query histories.
    """

    queries: typing.Optional[typing.List[QueryHistorySummary]] = None
    metadata: typing.Optional[ListQueryHistoriesResponseMetadata] = pydantic.Field(default=None)
    """
    The metadata for the list of query histories.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
