# This file was auto-generated by Fern from our API Definition.

from .create_client_credentials_request import CreateClientCredentialsRequest

ComponentsSchemasCreateClientCredentialsRequest = CreateClientCredentialsRequest
